<?php

namespace DHLParcel\Shipping\Model\Service;

use DHLParcel\Shipping\Model\Core\SingletonAbstract;
use DHLParcel\Shipping\Model\Data\Api\Request\Shipment as ShipmentRequest;

class ErrorFormatService extends SingletonAbstract
{
    public function formatCapabilitiesError(ShipmentRequest $request)
    {
        $toCountry = $request->receiver->address->countryCode;
        $toPostalCode = $request->receiver->address->postalCode;
        $toBusiness = $request->receiver->address->isBusiness;

        $fromCountry = $request->shipper->address->countryCode;

        $optionsDeliveryOptions = [];
        $shipmentRequestDeliveryOption = '';
        $shipmentRequestParcelSize = isset($request->pieces[0]) ? $request->pieces[0]->parcelType : '';
        foreach ($request->options as $option) {
            if ($option->key == 'DOOR' || $option->key == 'PS' || $option->key == 'BP') {
                $optionsDeliveryOptions[] = $option->key;
                $shipmentRequestDeliveryOption = $option->key;
            }
        }

        $optionsService = OptionService::instance();
        $capabilityResponse = CapabilityService::instance()->check($toCountry, $toPostalCode, $toBusiness, $optionsDeliveryOptions);

        $capabilityOptions = [];
        $shipmentRequestOptions = [];
        $unavailableOptions = [];
        $detectedExclusions = [];
        $excludedOptions = [];
        $deliveryOptionSizes = [];

        foreach ($request->options as $option) {
            $shipmentRequestOptions[] = $option->key;
        }

        if (!$capabilityResponse) {
            return null;
        }

        foreach ($capabilityResponse as $capability) {
            if (!isset($capability->options)) {
                continue;
            }

            foreach ($capability->options as $option) {
                if (!isset($option->key)) {
                    continue;
                }

                if ($capability->parcelType->key === $shipmentRequestParcelSize || $shipmentRequestParcelSize == '') {
                    $capabilityOptions[] = $option->key;
                }

                if ($option->optionType == 'DELIVERY_OPTION') {
                    if (!isset($deliveryOptionSizes[$option->key])) {
                        $deliveryOptionSizes[$option->key] = [];
                    }

                    if (!isset($deliveryOptionSizes[$option->key][$capability->parcelType->key])) {
                        $deliveryOptionSizes[$option->key][] = $capability->parcelType->key;
                    }
                }

                if ($capability->parcelType->key !== $shipmentRequestParcelSize && $shipmentRequestParcelSize != '') {
                    continue;
                }

                if (isset($option->exclusions)) {
                    foreach ($option->exclusions as $exclusion) {
                        $excludedOptions[] = $exclusion->key;

                        if (in_array($option->key, $shipmentRequestOptions) && in_array($exclusion->key, $shipmentRequestOptions)) {
                            if (!isset($detectedExclusions[$option->key])) {
                                $detectedExclusions[$option->key] = [];
                            }

                            if (!in_array($exclusion->key, $detectedExclusions[$option->key])) {
                                $detectedExclusions[$option->key][] = $exclusion->key;
                            }
                        }
                    }
                }
            }
        }

        array_unique($excludedOptions);

        foreach ($shipmentRequestOptions as $option) {
            if (!in_array($option, $capabilityOptions)) {
                $unavailableOptions[] = $option;
            }
        }

        $receiverTypeAddress = 'consumer';
        if ($toBusiness) {
            $receiverTypeAddress = 'business';
        }

        $servicesString = '';
        foreach ($shipmentRequestOptions as $option) {
            if (!array_key_exists($option, $deliveryOptionSizes)) {
                $servicesString .= $optionsService->getLabel($option) . ", ";
            }
        }
        $servicesString = substr(trim($servicesString), 0, -1);

        $capabilityMessagePrefix = <<<EOD
ERROR
 - Shipment could not be created. Please check the report below.

 REQUESTED
 - Sending from $fromCountry to $toCountry
 - Sending to a $receiverTypeAddress address
 - Delivery: $shipmentRequestDeliveryOption
 - Size: $shipmentRequestParcelSize
 - Services: $servicesString

 REASON:\n
EOD;
        $capabilityMessageReason = '';
        if (count($shipmentRequestOptions) === 0) {
            $capabilityMessageReason .= "- Missing delivery type\n";
        } elseif (count($deliveryOptionSizes) === 0) {
            $capabilityMessageReason .= "- This location has no available delivery methods\n";
        } elseif (!in_array($shipmentRequestDeliveryOption, array_keys($deliveryOptionSizes))) {
            $noDeliveryString = '';
            foreach (array_keys($deliveryOptionSizes) as $deliveryOptionKey) {
                $noDeliveryString .= "'" . $deliveryOptionKey . "', ";
            }
            $noDeliveryString = substr(trim($noDeliveryString), 0, -1);
            $capabilityMessageReason .= "- This location has $noDeliveryString, you've chosen '$shipmentRequestDeliveryOption delivery'\n";
        } elseif (in_array($shipmentRequestDeliveryOption, array_keys($deliveryOptionSizes))
            && !in_array($shipmentRequestParcelSize, $deliveryOptionSizes[$shipmentRequestDeliveryOption])
            && $shipmentRequestParcelSize != ''
        ) {
            // EXAMPLE
            // BP can only be combined with the size XSMALL, you've chosen SMALL
            $sizeCount = count($deliveryOptionSizes[$shipmentRequestDeliveryOption]);
            $sizeString = $sizeCount > 1 ? 'size' : 'sizes';
            $sizeListString = '';
            $i = 0;
            foreach ($deliveryOptionSizes[$shipmentRequestDeliveryOption] as $size) {
                $i++;
                if ($sizeCount > 1 && $i > 1) {
                    $sizeListString .= ($i == $sizeCount) ? ' and ' : ', ';
                }

                $sizeListString .= $size;
            }

            $capabilityMessageReason .= "- '$shipmentRequestDeliveryOption delivery' can only be used with the $sizeString $sizeListString, you've chosen $shipmentRequestParcelSize\n";
        } else {
            if (count($unavailableOptions) > 0) {
                foreach ($unavailableOptions as $option) {
                    $capabilityMessageReason .= "- '" . $optionsService->getLabel($option) . "' is not available for this location\n";
                }
            }

            if (count($detectedExclusions) > 0) {
                foreach ($detectedExclusions as $requestedOption => $excludedOptions) {
                    $excludedString = implode(", ", $excludedOptions);
                    foreach ($excludedOptions as $excludedOption) {
                        $excludedString = $optionsService->getLabel($excludedOption) . ', ';
                    }
                    $excludedString = substr(trim($excludedString), 0, -1);

                    $capabilityMessageReason .= "- '" . $optionsService->getLabel($requestedOption) . "' cannot be combined with '$excludedString'\n";
                }
            }
        }

        if (empty($capabilityMessageReason)) {
            return null;
        }

        return $capabilityMessagePrefix . $capabilityMessageReason;
    }
}
